#ifndef DUNE_CURVEDGRID_IDSET_HH
#define DUNE_CURVEDGRID_IDSET_HH

#include <type_traits>

#include <dune/grid/common/indexidset.hh>

namespace Dune {
namespace Curved {

// IdSet
// -----

template <class Grid, class HostIdSet>
class IdSet
    : public Dune::IdSet<Grid, IdSet<Grid, HostIdSet>, typename HostIdSet::IdType>
{
  using Self = IdSet;
  using Super = Dune::IdSet<Grid, Self, typename HostIdSet::IdType>;

  using Traits = typename std::remove_const<Grid>::type::Traits;

public:
  using IdType = typename HostIdSet::IdType;

public:
  IdSet () = default;

  explicit IdSet (const HostIdSet& hostIdSet)
    : hostIdSet_(&hostIdSet)
  {}

  /// \brief Copy constructor. Copies only the host-idset pointer
  IdSet (const IdSet& other)
    : hostIdSet_(other.hostIdSet_)
  {}

  /// \brief Copy-assignment operator. Copies only the host-idset pointer
  IdSet& operator= (const IdSet& other)
  {
    hostIdSet_ = other.hostIdSet_;
    return *this;
  }

  template <int codim>
  IdType id (const typename Traits::template Codim<codim>::Entity& entity) const
  {
    return hostIdSet().id(Grid::template getHostEntity<codim>(entity));
  }

  using Super::id;

  IdType subId (const typename Traits::template Codim<0>::Entity& entity, int i, unsigned int codim) const
  {
    return hostIdSet().subId(Grid::template getHostEntity<0>(entity), i, codim);
  }

  using Super::subId;

  explicit operator bool () const { return bool(hostIdSet_); }

private:
  const HostIdSet& hostIdSet () const
  {
    assert( *this );
    return *hostIdSet_;
  }

  const HostIdSet* hostIdSet_ = nullptr;
};

} // end namespace Curved
} // end namespace Dune

#endif // DUNE_CURVEDGRID_IDSET_HH
